package cn.iocoder.yudao.module.order.controller.admin.orderCargoControlPick;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.spring.enums.UserTypeEnum;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderCargoControlPick.OrderCargoControlPickDO;
import cn.iocoder.yudao.module.order.dto.OrderCargoControlReleaseInfoDto;
import cn.iocoder.yudao.module.order.service.orderCargoControl.OrderCargoControlService;
import cn.iocoder.yudao.module.order.service.orderCargoControlPick.OrderCargoControlPickService;
import cn.iocoder.yudao.module.order.vo.order.OrderBackPageVO;
import cn.iocoder.yudao.module.order.vo.order.OrderQueryVO;
import cn.iocoder.yudao.module.order.vo.order.StatisticsOrderVO;
import cn.iocoder.yudao.module.order.vo.orderCargoControlPick.OrderCargoControlPickApplyVO;
import cn.iocoder.yudao.module.order.vo.orderCargoControlPick.OrderCargoControlPickCreateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.ORDER_CARGO_CONTROL_PICK_NOT_EXISTS;

@Validated
@RestController
@Api(tags = "管理后台 - 订单控货人放货记录")
@RequestMapping("/ecw/order-cargo-control-pick")
public class OrderCargoControlPickController {

    @Resource
    private OrderCargoControlPickService orderCargoControlPickService;
    @Resource
    private OrderCargoControlService orderCargoControlService;


    /*
     控货订单详情列表
     */
    @GetMapping("/cargo/control/orderDetail")
    @ApiOperation("获得控货订单列表查询订单分页")
    public CommonResult<PageResult<OrderCargoControlReleaseInfoDto>> orderCargoControlDetailList(OrderQueryVO query, PageVO page) {
        PageResult<OrderCargoControlReleaseInfoDto> pageResult =
                orderCargoControlPickService.orderCargoControlDetailList(query, page);
        return success(pageResult);
    }

    @GetMapping("/cargo/control/statistics")
    @ApiOperation("查询控货订单列表统计")
    public CommonResult<StatisticsOrderVO> statisticsOrderControl(OrderQueryVO query) {
        return success(orderCargoControlPickService.statisticsOrderControl(query));
    }

    @PostMapping("/create")
    @ApiOperation("创建订单控货人放货记录")
    @Idempotent(timeout = 5)
    public CommonResult<Long> createOrderCargoControlPick(
            @Valid @RequestBody OrderCargoControlPickCreateReqVO createReqVO) {
        return success(orderCargoControlPickService.createOrderCargoControlPick(
                createReqVO));
    }


    @PutMapping("/review/{id}")
    @ApiOperation("复合订单控货人放货")
    public CommonResult<Boolean> reviewOrderCargoControlPick(@PathVariable("id") Long id) {
        OrderCargoControlPickDO orderCargoControlPickDO = orderCargoControlPickService.getById(id);
        if (Objects.isNull(orderCargoControlPickDO)) {
            throw exception(ORDER_CARGO_CONTROL_PICK_NOT_EXISTS);
        }
        orderCargoControlPickService.reviewOrderCargoControlPick(orderCargoControlPickDO);
        return success(Boolean.TRUE);
    }

    @PutMapping("/review/")
    @ApiOperation("批量复合订单控货人放货")
    @ApiImplicitParam(name = "orderIds", value = "订单ids(批量处理)", required = true, example = "1024,1025,1026", dataType = "String")
    public CommonResult<Boolean> reviewOrderCargoControlPick(String orderIds) {
        List<OrderCargoControlPickDO> orderCargoControlPickDOList = orderCargoControlPickService.list(new LambdaQueryWrapper<OrderCargoControlPickDO>()
                .in(OrderCargoControlPickDO::getOrderId, Arrays.asList(orderIds.split(StrUtil.COMMA)))
                .eq(OrderCargoControlPickDO::getStatus, 1));
        if (CollectionUtil.isEmpty(orderCargoControlPickDOList)) {
            throw exception(ORDER_CARGO_CONTROL_PICK_NOT_EXISTS);
        }
        orderCargoControlPickService.reviewOrderCargoControlPick(orderCargoControlPickDOList);
        return success(Boolean.TRUE);
    }

//    @PutMapping("/anti-checking")
//    @ApiOperation("反复合订单控货人放货")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "orderId", value = "订单ID(批量处理)",  example = "1024", dataType = "Long"),
//            @ApiImplicitParam(name = "id", value = "放货记录ID",  example = "1024", dataType = "Long")
//    })
//    @PreAuthorize("@ss.hasPermission('ecw:order-cargo-control-pick:update')")
//    public CommonResult<Boolean> antiCheckingOrderCargoControlPick(Long orderId, Long id) {
//        List<OrderCargoControlPickDO> allOrderCargoControlPickDOList = new ArrayList<>();
//        if (Objects.nonNull(orderId) && orderId > 0){
//            List<OrderCargoControlPickDO> orderCargoControlPickDOList = orderCargoControlPickService.list(new LambdaQueryWrapper<OrderCargoControlPickDO>()
//                    .eq(OrderCargoControlPickDO::getOrderId, orderId)
//                    .eq(OrderCargoControlPickDO::getStatus, 3)
//                    .eq(OrderCargoControlPickDO::getDeleted, Boolean.FALSE)
//            );
//            if (CollectionUtil.isEmpty(orderCargoControlPickDOList)) {
//                throw exception(ORDER_CARGO_CONTROL_PICK_NOT_EXISTS);
//            }
//            allOrderCargoControlPickDOList.addAll(orderCargoControlPickDOList);
//        }else {
//            OrderCargoControlPickDO orderCargoControlPickDO = orderCargoControlPickService.getById(id);
//            if (Objects.isNull(orderCargoControlPickDO)) {
//                throw exception(ORDER_CARGO_CONTROL_PICK_NOT_EXISTS);
//            }
//            orderId = orderCargoControlPickDO.getOrderId();
//            allOrderCargoControlPickDOList.add(orderCargoControlPickDO);
//        }
//        orderCargoControlPickService.batchAntiCheckingOrderCargoControlPick(allOrderCargoControlPickDOList, orderId);
//        return success(Boolean.TRUE);
//    }

    @PutMapping("/cancel/{id}")
    @ApiOperation("订单控货人取消放货")
    public CommonResult<Boolean> cancelOrderCargoControlPick(@PathVariable("id") Long id) {
        OrderCargoControlPickDO orderCargoControlPickDO = orderCargoControlPickService.getById(id);
        if (Objects.isNull(orderCargoControlPickDO)) {
            throw exception(ORDER_CARGO_CONTROL_PICK_NOT_EXISTS);
        }
        orderCargoControlPickService.cancelOrderCargoControlPick(orderCargoControlPickDO);
        return success(Boolean.TRUE);
    }


    @GetMapping("/release/info/{orderId}")
    @ApiOperation("获得控货订单放货详情")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OrderCargoControlReleaseInfoDto> getOrderCargoControlReleaseInfo(@PathVariable("orderId") Long orderId) {
        OrderCargoControlReleaseInfoDto releaseInfoDto = orderCargoControlPickService.getOrderCargoControlReleaseInfo(orderId);
        return success(releaseInfoDto);
    }


    @PutMapping("/update/apply")
    @ApiOperation("控货业务审批申请")
    public CommonResult<Boolean> orderCargoControlPickApply(@Valid @RequestBody OrderCargoControlPickApplyVO orderCargoControlPickApplyVO) {
        orderCargoControlPickService.orderCargoControlPickApply(orderCargoControlPickApplyVO);
        return success(true);
    }

    @GetMapping("update/info")
    @ApiOperation("获得控货订单放货修改申请详情")
    @ApiImplicitParam(name = "approveId", value = "审核ID", required = true, example = "1024", dataType = "Long")
    public CommonResult<OrderCargoControlPickApplyVO> approvalOrderCargoControlPick(@RequestParam("approveId") Long approveId) {
        OrderCargoControlPickApplyVO applyVO = orderCargoControlPickService.approvalOrderCargoControlPick(approveId);
        return success(applyVO);
    }



    @GetMapping("cancel/info/{id}")
    @ApiOperation("根据放货ID查询控货订单取消放货审核详情")
    @ApiImplicitParam(name = "id", value = "放货记录ID", required = true, example = "1024", dataType = "Long")
    public CommonResult<OrderCargoControlPickApplyVO> approvalCancelCargoControlPickByCargoControlPickId(@PathVariable(value = "id") Long id) {
        OrderCargoControlPickApplyVO applyVO = orderCargoControlPickService.approvalCancelCargoControlPickByCargoControlPickId(id);
        return success(applyVO);
    }

    @GetMapping("update/info/{id}")
    @ApiOperation("根据放货ID查询控货订单放货修改审核详情")
    @ApiImplicitParam(name = "id", value = "放货记录ID", required = true, example = "1024", dataType = "Long")
    public CommonResult<OrderCargoControlPickApplyVO> approvalUpdateCargoControlPickByCargoControlPickId(@PathVariable(value = "id") Long id) {
        OrderCargoControlPickApplyVO applyVO = orderCargoControlPickService.approvalUpdateCargoControlPickByCargoControlPickId(id);
        return success(applyVO);
    }

    @GetMapping("anti-checking/info/{orderId}")
    @ApiOperation("根据订单ID查询控货订单放货反复核审核详情")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataType = "Long")
    public CommonResult<OrderCargoControlPickApplyVO> approvalAntiCheckingOrderCargoControlPickByOrderId(@PathVariable(value = "orderId") Long orderId) {
        OrderCargoControlPickApplyVO applyVO = orderCargoControlPickService.approvalAntiCheckingOrderCargoControlPickByOrderId(orderId);
        return success(applyVO);
    }

    @GetMapping("seasoning/condiments/info/{cargoControlPickId}")
    @ApiOperation("根据放货ID查询调货申请业务审核详情")
    @ApiImplicitParam(name = "cargoControlPickId", value = "放货记录ID", required = true, example = "1024", dataType = "Long")
    public CommonResult<OrderCargoControlPickApplyVO> seasoningCondimentsCargoControlPickByCargoControlPickId(@PathVariable(value = "cargoControlPickId") Long cargoControlPickId) {
        OrderCargoControlPickApplyVO applyVO = orderCargoControlPickService.seasoningCondimentsOrderCargoControlPickByOrderId(cargoControlPickId);
        return success(applyVO);
    }

    @GetMapping("/seasoning-condiments/cargo-control/order/select")
    @ApiOperation("获得可调货的控货订单列表")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataType = "Long")
    public CommonResult<List<OrderBackPageVO>> seasoningCondimentsCargoControlOrderSelect(@RequestParam("orderId") Long orderId) {

        List<OrderBackPageVO> list = orderCargoControlService.seasoningCondimentsCargoControlOrderPage(orderId, UserTypeEnum.ADMIN.getValue());
        return success(list);
    }


    @GetMapping("cancel/approval/{orderApprovalId}")
    @ApiOperation("根据审批id取消审批")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderApprovalId", value = "审批ID", required = true, example = "1024", dataType = "Long"),
            @ApiImplicitParam(name = "reason", value = "取消原因", required = true, example = "1024", dataType = "String")
    })
    @Deprecated
    public CommonResult<Boolean> cancelApprovalByOrderApprovalId(@PathVariable(value = "orderApprovalId") Long orderApprovalId,
                                                                 @RequestParam(value = "reason") String reason) {
        orderCargoControlPickService.cancelApprovalByOrderApprovalId(orderApprovalId, reason);
        return success(Boolean.TRUE);
    }

    @GetMapping("create/customer")
    @ApiOperation("根据放货ID生成客户信息")
    @ApiImplicitParam(name = "CargoControlPickId", value = "放货记录ID", required = true, example = "1024", dataType = "Long")
    public CommonResult<Boolean> createCustomerByCargoControlPickId( Long CargoControlPickId) {
        OrderCargoControlPickDO orderCargoControlPickDO = orderCargoControlPickService.getById(CargoControlPickId);
        if (Objects.isNull(orderCargoControlPickDO)) {
            throw exception(ORDER_CARGO_CONTROL_PICK_NOT_EXISTS);
        }
        orderCargoControlPickService.createCustomerByCargoControlPickId(orderCargoControlPickDO);
        return success(true);
    }
}
