package cn.iocoder.yudao.module.order.controller.admin.orderWarehouseCheck;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseCheck.OrderWarehouseCheckDO;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.order.service.orderWarehouseCheck.OrderWarehouseCheckService;
import cn.iocoder.yudao.module.order.vo.order.OrderQueryVO;
import cn.iocoder.yudao.module.order.vo.orderWarehouseCheck.OrderForceCheck;
import cn.iocoder.yudao.module.order.vo.orderWarehouseCheck.OrderWarehouseCheckUpdateReqVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 -  订单到仓相关接口")
@RequestMapping("/order/order-warehouse-check")
public class OrderWarehouseCheckController {
    @Autowired
    private OrderWarehouseCheckService service;
    @Autowired
    private OrderQueryService orderQueryService;

    @GetMapping("/query")
    @ApiOperation("到仓查询")
    @ApiImplicitParam(name = "orderItemId", value = "订单项id", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<OrderWarehouseCheckDO>> query(@RequestParam("orderItemId") Long orderItemId) {
        return success(service.listByOrderItemId(orderItemId));
    }

    @PostMapping("/force")
    @ApiOperation("强制到仓")
    public CommonResult<Boolean> force(@Valid @RequestBody OrderForceCheck orderForceCheck) {
        service.force(orderForceCheck);
        return success(true);
    }

    @PostMapping("/revoke/{orderId}")
    @ApiOperation("撤销到仓")
    @ApiImplicitParam(name = "orderId", value = "订单id", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Boolean> revoke(@PathVariable("orderId") Long orderId) {
        service.revoke(orderId);
        return success(true);
    }

    @PutMapping("/update")
    @ApiOperation("到仓修改")
    public CommonResult<Boolean> update(@Valid @RequestBody OrderWarehouseCheckUpdateReqVO req) {
        service.update(req);
        return success(true);
    }


    @PutMapping("/all/force")
    @ApiOperation("全部强制到仓(紧刷新数据处理)")
    public CommonResult<Boolean> allForce() {
        service.allForce();
        return success(true);
    }
}
